@@ -3,14 +3,21 @@ import datetime
 import json
 import xmlrpclib
 
-from django.db import transaction
 from django.contrib.auth.models import Permission, User
+from django.test import TransactionTestCase
 from django.test.client import Client
 
 from django_testscenarios.ubertest import TestCase
 
-from lava_scheduler_app.models import Device, DeviceType, Tag, TestJob
+from linaro_django_xmlrpc.models import AuthToken
 
+from lava_scheduler_app.models import (
+    Device,
+    DeviceType,
+    JSONDataError,
+    Tag,
+    TestJob)
+from lava_scheduler_daemon.dbjobsource import DatabaseJobSource
 
 
 # Based on http://www.technobabble.dk/2008/apr/02/xml-rpc-dispatching-through-django-test-client/
@@ -26,9 +33,6 @@ class TestTransport(xmlrpclib.Transport):
         self._use_datetime = True
 
     def request(self, host, handler, request_body, verbose=0):
-        from django.conf import settings
-        # This is a total hack.  See bug 904054 for more.
-        settings.MOUNT_POINT = ''
         self.verbose = verbose
         response = self.client.post(
             handler, request_body, content_type="text/xml")
@@ -60,6 +64,9 @@ class ModelFactory(object):
             name = self.getUniqueString('name')
         return DeviceType.objects.get_or_create(name=name)[0]
 
+    def ensure_tag(self, name):
+        return Tag.objects.get_or_create(name=name)[0]
+
     def make_device(self, device_type=None, hostname=None):
         if device_type is None:
             device_type = self.ensure_device_type()
@@ -142,8 +149,16 @@ class TestTestJob(TestCaseWithFactory):
             self.factory.make_user())
         self.assertEqual(set(job.tags.all()), set([]))
 
+    def test_from_json_and_user_errors_on_unknown_tags(self):
+        self.factory.ensure_device_type(name='panda')
+        self.assertRaises(
+            JSONDataError, TestJob.from_json_and_user,
+            json.dumps({'device_type':'panda', 'device_tags':['unknown']}),
+            self.factory.make_user())
+
     def test_from_json_and_user_sets_tag_from_device_tags(self):
         self.factory.ensure_device_type(name='panda')
+        self.factory.ensure_tag('tag')
         job = TestJob.from_json_and_user(
             json.dumps({'device_type':'panda', 'device_tags':['tag']}),
             self.factory.make_user())
@@ -152,6 +167,8 @@ class TestTestJob(TestCaseWithFactory):
 
     def test_from_json_and_user_sets_multiple_tag_from_device_tags(self):
         self.factory.ensure_device_type(name='panda')
+        self.factory.ensure_tag('tag1')
+        self.factory.ensure_tag('tag2')
         job = TestJob.from_json_and_user(
             json.dumps({'device_type':'panda', 'device_tags':['tag1', 'tag2']}),
             self.factory.make_user())
@@ -160,6 +177,7 @@ class TestTestJob(TestCaseWithFactory):
 
     def test_from_json_and_user_reuses_tag_objects(self):
         self.factory.ensure_device_type(name='panda')
+        self.factory.ensure_tag('tag')
         job1 = TestJob.from_json_and_user(
             json.dumps({'device_type':'panda', 'device_tags':['tag']}),
             self.factory.make_user())
@@ -240,10 +258,6 @@ class TestSchedulerAPI(TestCaseWithFactory):
         self.assertEqual(TestJob.CANCELED, job.status)
 
 
-from django.test import TransactionTestCase
-
-from lava_scheduler_daemon.dbjobsource import DatabaseJobSource
-
 class TransactionTestCaseWithFactory(TransactionTestCase):
 
     def setUp(self):
@@ -251,26 +265,32 @@ class TransactionTestCaseWithFactory(TransactionTestCase):
         self.factory = ModelFactory()
 
 
+class NonthreadedDatabaseJobSource(DatabaseJobSource):
+    deferToThread = staticmethod(lambda f, *args, **kw: f(*args, **kw))
+
+
 class TestDBJobSource(TransactionTestCaseWithFactory):
 
+    def setUp(self):
+        super(TestDBJobSource, self).setUp()
+        self.source = NonthreadedDatabaseJobSource()
+
     def test_getBoardList(self):
         self.factory.make_device(hostname='panda01')
-        self.assertEqual(['panda01'], DatabaseJobSource().getBoardList_impl())
+        self.assertEqual(['panda01'], self.source.getBoardList())
 
     def test_getJobForBoard_returns_json(self):
         device = self.factory.make_device(hostname='panda01')
         definition = {'foo': 'bar', 'target': 'panda01'}
         self.factory.make_testjob(
             requested_device=device, definition=json.dumps(definition))
-        transaction.commit()
         self.assertEqual(
-            definition, DatabaseJobSource().getJobForBoard_impl('panda01'))
+            definition, self.source.getJobForBoard('panda01'))
 
     def test_getJobForBoard_returns_None_if_no_job(self):
         self.factory.make_device(hostname='panda01')
-        transaction.commit()
         self.assertEqual(
-            None, DatabaseJobSource().getJobForBoard_impl('panda01'))
+            None, self.source.getJobForBoard('panda01'))
 
     def test_getJobForBoard_considers_device_type(self):
         panda_type = self.factory.ensure_device_type(name='panda')
@@ -279,10 +299,9 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         self.factory.make_testjob(
             requested_device_type=panda_type,
             definition=json.dumps(definition))
-        transaction.commit()
         definition['target'] = 'panda01'
         self.assertEqual(
-            definition, DatabaseJobSource().getJobForBoard_impl('panda01'))
+            definition, self.source.getJobForBoard('panda01'))
 
     def test_getJobForBoard_prefers_older(self):
         panda_type = self.factory.ensure_device_type(name='panda')
@@ -296,10 +315,9 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         self.factory.make_testjob(
             requested_device=panda01, definition=json.dumps(second_definition),
             submit_time=datetime.datetime.now())
-        transaction.commit()
         self.assertEqual(
             first_definition,
-            DatabaseJobSource().getJobForBoard_impl('panda01'))
+            self.source.getJobForBoard('panda01'))
 
     def test_getJobForBoard_prefers_directly_targeted(self):
         panda_type = self.factory.ensure_device_type(name='panda')
@@ -314,10 +332,9 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         self.factory.make_testjob(
             requested_device=panda01,
             definition=json.dumps(device_definition))
-        transaction.commit()
         self.assertEqual(
             device_definition,
-            DatabaseJobSource().getJobForBoard_impl('panda01'))
+            self.source.getJobForBoard('panda01'))
 
     def test_getJobForBoard_avoids_targeted_to_other_board_of_same_type(self):
         panda_type = self.factory.ensure_device_type(name='panda')
@@ -328,10 +345,9 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         self.factory.make_testjob(
             requested_device=panda01,
             definition=json.dumps(definition))
-        transaction.commit()
         self.assertEqual(
             None,
-            DatabaseJobSource().getJobForBoard_impl('panda02'))
+            self.source.getJobForBoard('panda02'))
 
     def _makeBoardWithTags(self, tags):
         board = self.factory.make_device()
@@ -350,14 +366,14 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         self._makeJobWithTagsForBoard(job_tags, board)
         self.assertEqual(
             board.hostname,
-            DatabaseJobSource().getJobForBoard_impl(board.hostname)['target'])
+            self.source.getJobForBoard(board.hostname)['target'])
 
     def assertBoardWithTagsDoesNotGetJobWithTags(self, board_tags, job_tags):
         board = self._makeBoardWithTags(board_tags)
         self._makeJobWithTagsForBoard(job_tags, board)
         self.assertEqual(
             None,
-            DatabaseJobSource().getJobForBoard_impl(board.hostname))
+            self.source.getJobForBoard(board.hostname))
 
     def test_getJobForBoard_does_not_return_job_if_board_lacks_tag(self):
         self.assertBoardWithTagsDoesNotGetJobWithTags([], ['tag'])
@@ -384,8 +400,7 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(requested_device=device)
         before = datetime.datetime.now()
-        transaction.commit()
-        DatabaseJobSource().getJobForBoard_impl('panda01')
+        self.source.getJobForBoard('panda01')
         after = datetime.datetime.now()
         # reload from the database
         job = TestJob.objects.get(pk=job.pk)
@@ -394,8 +409,7 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
     def test_getJobForBoard_set_statuses(self):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(requested_device=device)
-        transaction.commit()
-        DatabaseJobSource().getJobForBoard_impl('panda01')
+        self.source.getJobForBoard('panda01')
         # reload from the database
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
@@ -406,24 +420,91 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
     def test_getJobForBoard_sets_running_job(self):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(requested_device=device)
-        transaction.commit()
-        DatabaseJobSource().getJobForBoard_impl('panda01')
+        self.source.getJobForBoard('panda01')
         # reload from the database
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(job, device.current_job)
 
+    def test_getJobForBoard_creates_token(self):
+        device = self.factory.make_device(hostname='panda01')
+        job = self.factory.make_testjob(requested_device=device)
+        self.source.getJobForBoard('panda01')
+        # reload from the database
+        job = TestJob.objects.get(pk=job.pk)
+        device = Device.objects.get(pk=device.pk)
+        self.assertIsNotNone(job.submit_token)
+        self.assertEqual(job.submitter, job.submit_token.user)
+
+    def test_getJobForBoard_inserts_target_into_json(self):
+        panda_type = self.factory.ensure_device_type(name='panda')
+        self.factory.make_device(hostname='panda01', device_type=panda_type)
+        definition = {'foo': 'bar'}
+        self.factory.make_testjob(
+            requested_device_type=panda_type,
+            definition=json.dumps(definition))
+        json_data = self.source.getJobForBoard('panda01')
+        self.assertIn('target', json_data)
+        self.assertEqual('panda01', json_data['target'])
+
+    def test_getJobForBoard_inserts_submit_token_into_json(self):
+        panda_type = self.factory.ensure_device_type(name='panda')
+        self.factory.make_device(hostname='panda01', device_type=panda_type)
+        definition = {
+            'actions': [
+                {
+                    "command": "submit_results",
+                    "parameters":
+                    {
+                        "server": "http://test-server/RPC2/",
+                        "stream": "/private/personal/test/test/",
+                    }
+                }
+            ]
+        }
+        job = self.factory.make_testjob(
+            requested_device_type=panda_type,
+            definition=json.dumps(definition))
+        json_data = self.source.getJobForBoard('panda01')
+        job = TestJob.objects.get(pk=job.pk)
+        submit_job_params = json_data['actions'][0]['parameters']
+        self.assertIn('token', submit_job_params)
+        self.assertEqual(job.submit_token.secret, submit_job_params['token'])
+
+    def test_getJobForBoard_adds_user_to_url(self):
+        panda_type = self.factory.ensure_device_type(name='panda')
+        self.factory.make_device(hostname='panda01', device_type=panda_type)
+        user = User.objects.create_user('test', 'e@mail.invalid', 'test')
+        user.save()
+        definition = {
+            'actions': [
+                {
+                    "command": "submit_results",
+                    "parameters":
+                    {
+                        "server": "http://test-server/RPC2/",
+                        "stream": "/private/personal/test/test/",
+                    }
+                }
+            ]
+        }
+        job = self.factory.make_testjob(
+            requested_device_type=panda_type, submitter=user,
+            definition=json.dumps(definition))
+        json_data = self.source.getJobForBoard('panda01')
+        job = TestJob.objects.get(pk=job.pk)
+        submit_job_params = json_data['actions'][0]['parameters']
+        self.assertEqual("http://test@test-server/RPC2/", submit_job_params['server'])
+
     def get_device_and_running_job(self):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(requested_device=device)
-        transaction.commit()
-        DatabaseJobSource().getJobForBoard_impl('panda01')
-        return device, job
+        self.source.getJobForBoard('panda01')
+        return device, TestJob.objects.get(pk=job.pk)
 
     def test_jobCompleted_set_statuses_success(self):
         device, job = self.get_device_and_running_job()
-        transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01', 0)
+        self.source.jobCompleted('panda01', 0)
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(
@@ -432,8 +513,7 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
 
     def test_jobCompleted_set_statuses_failure(self):
         device, job = self.get_device_and_running_job()
-        transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01', 1)
+        self.source.jobCompleted('panda01', 1)
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(
@@ -444,9 +524,8 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
         device = self.factory.make_device(hostname='panda01')
         job = self.factory.make_testjob(
             requested_device_type=device.device_type)
-        transaction.commit()
-        DatabaseJobSource().getJobForBoard_impl('panda01')
-        DatabaseJobSource().jobCompleted_impl('panda01', 0)
+        self.source.getJobForBoard('panda01')
+        self.source.jobCompleted('panda01', 0)
         job = TestJob.objects.get(pk=job.pk)
         device = Device.objects.get(pk=device.pk)
         self.assertEqual(
@@ -456,25 +535,30 @@ class TestDBJobSource(TransactionTestCaseWithFactory):
     def test_jobCompleted_sets_end_time(self):
         device, job = self.get_device_and_running_job()
         before = datetime.datetime.now()
-        transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01', 0)
+        self.source.jobCompleted('panda01', 0)
         after = datetime.datetime.now()
         job = TestJob.objects.get(pk=job.pk)
         self.assertTrue(before < job.end_time < after)
 
     def test_jobCompleted_clears_current_job(self):
         device, job = self.get_device_and_running_job()
-        transaction.commit()
-        DatabaseJobSource().jobCompleted_impl('panda01', 0)
+        self.source.jobCompleted('panda01', 0)
         device = Device.objects.get(pk=device.pk)
         self.assertEquals(None, device.current_job)
 
+    def test_jobCompleted_deletes_token(self):
+        device, job = self.get_device_and_running_job()
+        token = job.submit_token
+        self.source.jobCompleted('panda01', 0)
+        self.assertRaises(
+            AuthToken.DoesNotExist,
+            AuthToken.objects.get, pk=token.pk)
+
     def test_getLogFileForJobOnBoard_returns_writable_file(self):
         device, job = self.get_device_and_running_job()
         definition = {'foo': 'bar'}
         self.factory.make_testjob(
             requested_device=device, definition=json.dumps(definition))
-        transaction.commit()
-        log_file = DatabaseJobSource().getLogFileForJobOnBoard_impl('panda01')
+        log_file = self.source.getLogFileForJobOnBoard('panda01')
         log_file.write('a')
         log_file.close()
