@@ -4,18 +4,24 @@ import os
 import simplejson
 import StringIO
 
-
+from django.conf import settings
+from django.core.urlresolvers import reverse
 from django.http import (
     HttpResponse,
+    HttpResponseBadRequest,
     HttpResponseForbidden,
     HttpResponseNotAllowed,
     )
-from django.template import RequestContext
 from django.shortcuts import (
     get_object_or_404,
     redirect,
     render_to_response,
 )
+from django.template import RequestContext
+from django.template import defaultfilters as filters
+
+from lava.utils.data_tables.views import DataTableView
+from lava.utils.data_tables.backends import QuerySetBackend, Column
 
 from lava_server.views import index as lava_index
 from lava_server.bread_crumbs import (
@@ -28,7 +34,12 @@ from lava_scheduler_app.logfile_helper import (
     getDispatcherErrors,
     getDispatcherLogMessages
     )
-from lava_scheduler_app.models import Device, TestJob
+from lava_scheduler_app.models import (
+    Device,
+    DeviceStateTransition,
+    TestJob,
+    )
+
 
 
 def post_only(func):
@@ -59,14 +70,58 @@ def job_list(request):
     return render_to_response(
         "lava_scheduler_app/alljobs.html",
         {
-            'jobs': TestJob.objects.select_related(
-                "actual_device", "requested_device", "requested_device_type",
-                "submitter").all(),
             'bread_crumb_trail': BreadCrumbTrail.leading_to(job_list),
         },
         RequestContext(request))
 
 
+def device_callback(job):
+    if job.actual_device:
+        return dict(
+            name=job.actual_device.pk, requested=False,
+            link=reverse(device_detail, kwargs=dict(pk=job.actual_device.pk)))
+    elif job.requested_device:
+        return dict(
+            name=job.requested_device.pk, requested=True,
+            link=reverse(device_detail, kwargs=dict(pk=job.requested_device.pk)))
+    else:
+        return dict(name=job.requested_device_type.pk, requested=True)
+
+
+def id_callback(job):
+    if job is None:
+        return job
+    else:
+        return dict(id=job.id, link=reverse(job_detail, kwargs=dict(pk=job.id)))
+
+
+alljobs_json = DataTableView.as_view(
+    backend=QuerySetBackend(
+        queryset=TestJob.objects.select_related(
+            "actual_device", "requested_device", "requested_device_type",
+            "submitter").extra(
+            select={
+                'device_sort': 'coalesce(actual_device_id, requested_device_id, requested_device_type_id)'
+                }).all(),
+        columns=[
+            Column(
+                'id', 'id', id_callback),
+            Column(
+                'status', 'status', lambda job: job.get_status_display()),
+            Column(
+                'device', 'device_sort', device_callback),
+            Column(
+                'description', 'description', lambda job: job.description),
+            Column(
+                'submitter', 'submitter', lambda job: job.submitter.username),
+            Column(
+                'submit_time', 'submit_time',
+                lambda job: filters.date(
+                    job.submit_time, settings.DATETIME_FORMAT)),
+            ],
+        searching_columns=['description']))
+
+
 @BreadCrumb("Job #{pk}", parent=index, needs=['pk'])
 def job_detail(request, pk):
     job = get_object_or_404(TestJob, pk=pk)
@@ -183,7 +238,11 @@ NEWLINE_SCAN_SIZE = 80
 
 
 def job_output(request, pk):
-    start = int(request.GET.get('start', 0))
+    start = request.GET.get('start', 0)
+    try:
+        start = int(start)
+    except ValueError:
+        return HttpResponseBadRequest("invalid start")
     count_present = 'count' in request.GET
     job = get_object_or_404(TestJob, pk=pk)
     log_file = job.log_file
@@ -239,10 +298,32 @@ def job_json(request, pk):
 @BreadCrumb("Device {pk}", parent=index, needs=['pk'])
 def device_detail(request, pk):
     device = get_object_or_404(Device, pk=pk)
+    if device.status in [Device.OFFLINE, Device.OFFLINING]:
+        try:
+            transition = device.transitions.filter(message__isnull=False).latest('created_on').message
+        except DeviceStateTransition.DoesNotExist:
+            transition = None
+    else:
+        transition = None
+    transition_models = device.transitions.order_by('created_on').select_related('created_by')
+    transition_list = []
+    if transition_models:
+        for i, t in enumerate(transition_models):
+            if i > 0:
+                before = transition_models[i-1].created_on
+            else:
+                before = None
+            transition_list.append(
+                (t.created_on, before,
+                 t.get_old_state_display(), t.get_new_state_display(),
+                 t.created_by, t.message))
+        transition_list.reverse()
     return render_to_response(
         "lava_scheduler_app/device.html",
         {
             'device': device,
+            'transition': transition,
+            'transition_list': transition_list,
             'recent_job_list': device.recent_jobs,
             'show_maintenance': device.can_admin(request.user) and \
                 device.status in [Device.IDLE, Device.RUNNING],
@@ -257,7 +338,7 @@ def device_detail(request, pk):
 def device_maintenance_mode(request, pk):
     device = Device.objects.get(pk=pk)
     if device.can_admin(request.user):
-        device.put_into_maintenance_mode()
+        device.put_into_maintenance_mode(request.user, request.POST.get('reason'))
         return redirect(device)
     else:
         return HttpResponseForbidden(
@@ -268,7 +349,7 @@ def device_maintenance_mode(request, pk):
 def device_online(request, pk):
     device = Device.objects.get(pk=pk)
     if device.can_admin(request.user):
-        device.put_into_online_mode()
+        device.put_into_online_mode(request.user, request.POST.get('reason'))
         return redirect(device)
     else:
         return HttpResponseForbidden(
